﻿using Autofac;
using Autofac.Extensions.DependencyInjection;
using Business.Layer;
using DataAccess.Layer;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using System;

namespace ASPNETWEBAPI2.Client
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.


        //public void ConfigureServices(IServiceCollection services)
        //{
        //    services.AddMvc();
        //    //Register application & data  services
        //    ConfigureServicesTransientLifetime(services);
        //    //ConfigureServicesSingletonLifetime(services);
        //    //ConfigureServicesScopedLifetime(services);
        //}
        public IServiceProvider ConfigureServices(IServiceCollection services)
        {
            services.AddMvc();

            //Create Autofac ContainerBuilder 
            var containerBuilder = new ContainerBuilder();
            var container = ConfigureServicesInstancePerDependency(containerBuilder, services);
            //var container = ConfigureServicesSingleInstance(containerBuilder, services);
            //var container = ConfigureServicesInstancePerLifetimeScope(containerBuilder, services);
            //var container = ConfigureServicesInstancePerMatchingLifetimeScope(containerBuilder, services);
            //var container = ConfigureServicesInstancePerRequest(containerBuilder, services);
            //var container = ConfigureServicesInstancePerOwned(containerBuilder, services);
            //var container = ConfigureServicesThreadScope(containerBuilder, services);
            //Create Autofac Service Provider & assign Autofac container 
            var autofacServiceProvider = new AutofacServiceProvider(container);
            //Finally return Autofack Service Provider.
            return autofacServiceProvider;
        }

        private IContainer ConfigureServicesInstancePerDependency(ContainerBuilder containerBuilder, IServiceCollection services)
        {
            //Register your module here.
            var connectionString = "MyConnectionString";
            var namedParameter = new NamedParameter("connectionString", connectionString);
            containerBuilder.RegisterType<DataRepository>().As<IDataRepository>()
                .WithParameter(namedParameter).InstancePerDependency();
            containerBuilder.RegisterType<DataService>().As<IDataService>().InstancePerDependency();
            //Plug-In your Autofac builder in ASP.NET Core PIPLINE.
            containerBuilder.Populate(services);
            //Build Autofac container.
            var container = containerBuilder.Build();

            var dataRepositoryObjectOne = container.Resolve<IDataRepository>();
            var dataRepositoryObjectTwo = container.Resolve<IDataRepository>();
            var result = object.ReferenceEquals(dataRepositoryObjectOne, dataRepositoryObjectTwo);

            return container;
        }
        private IContainer ConfigureServicesSingleInstance(ContainerBuilder containerBuilder, IServiceCollection services)
        {
            //Register your module here.
            var connectionString = "MyConnectionString";
            var namedParameter = new NamedParameter("connectionString", connectionString);
            containerBuilder.RegisterType<DataRepository>().As<IDataRepository>()
                .WithParameter(namedParameter).SingleInstance();
            containerBuilder.RegisterType<DataService>().As<IDataService>().SingleInstance();
            //Plug-In your Autofac builder in ASP.NET Core PIPLINE.
            containerBuilder.Populate(services);
            //Build Autofac container.
            var container = containerBuilder.Build();

            var dataRepositoryObjectOne = container.Resolve<IDataRepository>();
            var dataRepositoryObjectTwo = container.Resolve<IDataRepository>();
            var result = object.ReferenceEquals(dataRepositoryObjectOne, dataRepositoryObjectTwo);

            return container;
        }
        private IContainer ConfigureServicesInstancePerLifetimeScope(ContainerBuilder containerBuilder, IServiceCollection services)
        {
            //Register your module here.
            var connectionString = "MyConnectionString";
            var namedParameter = new NamedParameter("connectionString", connectionString);
            containerBuilder.RegisterType<DataRepository>().As<IDataRepository>()
                .WithParameter(namedParameter).InstancePerLifetimeScope();
            containerBuilder.RegisterType<DataService>().As<IDataService>().InstancePerLifetimeScope();
            //Plug-In your Autofac builder in ASP.NET Core PIPLINE.
            containerBuilder.Populate(services);
            //Build Autofac container.
            var container = containerBuilder.Build();

            IDataRepository dataRepositoryObjectOne;
            IDataRepository dataRepositoryObjectTwo;

            using (var scope = container.BeginLifetimeScope())
            {
                dataRepositoryObjectOne = scope.Resolve<IDataRepository>();

                IDataService dataServiceObjectOne = scope.Resolve<IDataService>();
                IDataService dataServiceObjectTwo = null;
                bool dataServiceObjectResult;

                for (var i = 0; i < 5; i++)
                {
                    dataServiceObjectTwo = scope.Resolve<IDataService>();
                    dataServiceObjectResult = object.ReferenceEquals(dataServiceObjectOne, dataServiceObjectTwo);
                }

                using (var childScope = container.BeginLifetimeScope())
                {
                    dataServiceObjectTwo = childScope.Resolve<IDataService>();
                    dataServiceObjectResult = object.ReferenceEquals(dataServiceObjectOne, dataServiceObjectTwo);
                }
            }

            using (var scope = container.BeginLifetimeScope())
            {
                dataRepositoryObjectTwo = scope.Resolve<IDataRepository>();
            }

            var result = object.ReferenceEquals(dataRepositoryObjectOne, dataRepositoryObjectTwo);

            return container;
        }
        private IContainer ConfigureServicesInstancePerMatchingLifetimeScope(ContainerBuilder containerBuilder, IServiceCollection services)
        {
            //Register your module here.
            var connectionString = "MyConnectionString";
            var namedParameter = new NamedParameter("connectionString", connectionString);
            containerBuilder.RegisterType<DataRepository>().As<IDataRepository>()
              .WithParameter(namedParameter).InstancePerMatchingLifetimeScope("WebAPI2.0Request");
            containerBuilder.RegisterType<DataService>().As<IDataService>().InstancePerMatchingLifetimeScope("WebAPI2.0Request");
            //Plug-In your Autofac builder in ASP.NET Core PIPLINE.
            containerBuilder.Populate(services);
            //Build Autofac container.
            var container = containerBuilder.Build();

            IDataRepository dataRepositoryObjectOne;
            IDataRepository dataRepositoryObjectTwo;

            using (var scope = container.BeginLifetimeScope("WebAPI2.0Request"))
            {
                dataRepositoryObjectOne = scope.Resolve<IDataRepository>();
                using (var childScope = scope.BeginLifetimeScope())
                {
                    dataRepositoryObjectTwo = childScope.Resolve<IDataRepository>();
                    var dataRepositoryObjectResult = object.ReferenceEquals(dataRepositoryObjectOne, dataRepositoryObjectTwo);

                    IDataService dataServiceObjectOne = scope.Resolve<IDataService>();
                    IDataService dataServiceObjectTwo = scope.Resolve<IDataService>();
                    bool dataServiceObjectResult = object.ReferenceEquals(dataServiceObjectOne, dataServiceObjectTwo);
                }

            }

            using (var scope = container.BeginLifetimeScope("WebAPI2.0Request"))
            {
                dataRepositoryObjectTwo = scope.Resolve<IDataRepository>();
            }

            var result = object.ReferenceEquals(dataRepositoryObjectOne, dataRepositoryObjectTwo);
            // This will throw an exception because this scope doesn't
            // have any expected BeginLifetimeScope tag and neither does any parent scope!
            //using (var scope = container.BeginLifetimeScope())
            //{
            //    dataRepositoryObjectTwo = scope.Resolve<IDataRepository>();
            //}
            return container;
        }
        private IContainer ConfigureServicesInstancePerRequest(ContainerBuilder containerBuilder, IServiceCollection services)
        {
            //Register your module here.
            //containerBuilder.RegisterModule<AutofacInstancePerDependencyModule>();
            //Above lines are equals to below lines because Instance Per Dependency is default lifecycle.
            //Also called 'transient or 'factory' in other containers. 
            var connectionString = "MyConnectionString";
            var namedParameter = new NamedParameter("connectionString", connectionString);
            containerBuilder.RegisterType<DataRepository>().As<IDataRepository>()
                .WithParameter(namedParameter).InstancePerRequest("WebAPI2.0Request");
            containerBuilder.RegisterType<DataService>().As<IDataService>().InstancePerRequest("WebAPI2.0Request");
            //Plug-In your Autofac builder in ASP.NET Core PIPLINE.
            containerBuilder.Populate(services);
            //Build Autofac container.
            var container = containerBuilder.Build();

            IDataRepository dataRepositoryObjectOne;
            IDataRepository dataRepositoryObjectTwo;

            using (var scope = container.BeginLifetimeScope("WebAPI2.0Request"))
            {
                dataRepositoryObjectOne = scope.Resolve<IDataRepository>();
                using (var childScope = scope.BeginLifetimeScope())
                {
                    dataRepositoryObjectTwo = childScope.Resolve<IDataRepository>();

                    IDataService dataServiceObjectOne = childScope.Resolve<IDataService>();
                    IDataService dataServiceObjectTwo = childScope.Resolve<IDataService>();

                    var dataServiceObjectTwoResult = object.ReferenceEquals(dataServiceObjectOne, dataServiceObjectTwo);

                }
                var dataRepositoryObjectResult = object.ReferenceEquals(dataRepositoryObjectOne, dataRepositoryObjectTwo);
            }

            using (var scope = container.BeginLifetimeScope("WebAPI2.0Request"))
            {
                dataRepositoryObjectTwo = scope.Resolve<IDataRepository>();
            }

            var result = object.ReferenceEquals(dataRepositoryObjectOne, dataRepositoryObjectTwo);
          
            return container;
        }
        private IContainer ConfigureServicesInstancePerOwned(ContainerBuilder containerBuilder, IServiceCollection services)
        {
            return containerBuilder.Build();
        }

        private IContainer ConfigureServicesThreadScope(ContainerBuilder containerBuilder, IServiceCollection services)
        {
            return containerBuilder.Build();
        }


        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseMvc();
        }
    }
}
